/* gil_resize.cpp
*  @author Rafa Oracz (R.Oracz@stud.elka.pw.edu.pl)
*
* Program demonstruje sposb zmiany rozmiarw obrazka,przy pomocy biblioteki GIL, i jej rozszerze: numeric, opencv.
* Zapis plikw wspiera rozszerzenie io. 
* W przykadzie posuguj si plikami typu jpg, do obsugi ktrych wymagana jest biblioteka libjpeg.lib (http://www.ijg.org)
  Do poprawnego dziaania musimy zmienic nazwe odpowieniego pliku jconfig.* na jconfig.h (dla VS: jconfig.vs -> jconfig.h)
*
* Zmiane rozmiarw moemy zrealizowa albo za pomoc rozszerzenia numeric, albo zapomoc rozszerzenia opencv.
* numeric: http://opensource.adobe.com/wiki/display/gil/Downloads
* opencv: http://sourceforge.net/projects/opencvlibrary/ + help: http://opencv.willowgarage.com/wiki/
*
*
* Dla poprawnej kompilacji musimy linkerowi wskaza odpowiednie katalogi: 
	/sciezka_do/OpenCV/lib  (jeli korzystamy z opencv)
	/sciezka_do/jpeg-6b/lib (obowizkowo)
  Oraz doda dodatkowe zalenosci:	
	Libjpeg: libjpeg.lib 
	OpenCV: cv.lib cxcore.lib (jeli korzystamy z opencv)
  Dodajemy cieki do niezbdnych nagwkw:
    /sciezka_do/OpenCV/cxcore/include	
    /sciezka_do/OpenCV/CV/include
    /sciezka_do/jpeg-6b
    /sciezka_do/boost_1_*  ;)
  Zignorowa biblioteke (Ignore Specific Library w 'Project properties->Linker->Input' dla VS)
	LIBC.lib
	
* Do testowania naley umiecic plik *.jpg w katalogu z progamem. Wywoanie opisane jest przy funkcji main. 	
* W wyniku dziaania programu, powstanie 6 nowych plikw obrazujcych dziaanie poszczeglnych sposobw zmiany rozmiaru.
*/

#include <iostream>
#include <boost/gil/image.hpp>
#include <boost/gil/typedefs.hpp>
#include <boost/gil/extension/io/jpeg_io.hpp>
#include <boost/gil/extension/numeric/sampler.hpp>
#include <boost/gil/extension/numeric/resample.hpp>
#include <boost/gil/extension/opencv/resize.hpp>

using namespace boost::gil;

int main(int argc, char* argv[]) // Wywoanie: gil_resize nazwa_pliku szeroko wysoko.
{
    if(argc != 4) { std::cout << "Usage: resize filename width height.\n"; return 1; };

    
	char* inputFileName = argv[1];				// nazwa pliku wejciowego w formacie jpg.
    int width = atoi(argv[2]);					// szeroko obrazu wyjciowego
    int height = atoi(argv[3]);					// wysoko obraz wyjciowego
    
    rgb8_image_t imgOriginal;					// 8-bitowy wejsciowy obraz RGB
    rgb8_image_t imgResized(width,height);		// 8-bitowy wyjsciowy obraz RGB
    
    
    /* Funkcje view(), const_view() zwracaj swojego rodzaju wskanik na obraz znajdujcy si w pamici.
		Jak mona si domyli funkcja const_view() zabezpiecza nas przed zmian obrazu, 
		dlatego dla obrazu wyjciowego bdziemy uzywa view(), dla oryginalnego const_view(). */
    
    
    
   	// ZACZYNAMY
   	// Na pocztek 2 przykady wykorzystujce rozszerzenie numeric dostarczajce 2 samplerw.	 
    
    
    /* Alokujemy pami zalen od wymiarw pliku wejciowego, nastpnie wgrana zostanie zawarto obrazka.
       Nstpuje tu rwnie sprawdzanie czy plik jest poprawnym formatem jpg, 
       i czy jego parametry s zgodne z parametrami imgOriginal. W razie bdu rzucany jest wyjtek std::ios_base::failure */
    jpeg_read_image(inputFileName, imgOriginal);  
    
		/* Funkcja resize_view odpowiedzialna jest za skopiowanie zawartoci obrazka, 
		   skalujc go do nowych rozmiarw, za pomoc samplera podanego jako 3ci argument */
		resize_view(const_view(imgOriginal), view(imgResized), bilinear_sampler()); 
		/* Zapisujemy otrzymany obraz pod wskazan nazw. 
		   Moe zosta rzucony wyjtek std::ios_base::failure
		   jeli format nie jest wspierany przez rozszerzenie I/O
		   lub zapisanie pliku si nie powidedzie.	 */
		jpeg_write_view("out.numeric.bilinear.jpg", const_view(imgResized));


	// powtarzamy dla nearest_neighbor_sampler:
		resize_view(const_view(imgOriginal), view(imgResized), nearest_neighbor_sampler());
		jpeg_write_view("out.numeric.nearest_neighbor.jpg", const_view(imgResized)); 
    
    
    
    
    // Teraz czas na rozszerzenie opencv, ktre dostarcza nam 4 rne algorytmy resamplingu.

		// Funkcja resize analogiczna do resize_view z numeric extension.
		opencv::resize(view(imgOriginal), view(imgResized), opencv::bilinear());
		jpeg_write_view("out.opencv.bilinear.jpg", const_view(imgResized));
		
		opencv::resize(view(imgOriginal), view(imgResized), opencv::nearest_neigbor()); // UWAGA! nazwa to nearest_neigbor, a nie nearest_neighbor. Bd jzykowy ??
		jpeg_write_view("out.opencv.nearest_neighbor.jpg", const_view(imgResized));
    
		opencv::resize(view(imgOriginal), view(imgResized), opencv::area());
		jpeg_write_view("out.opencv.area.jpg", const_view(imgResized));
	
		opencv::resize(view(imgOriginal), view(imgResized), opencv::bicubic());
		jpeg_write_view("out.opencv.bicubic.jpg", const_view(imgResized));

    return 0;
}
